<?php

/**
 * webEdition CMS
 *
 * $Rev: 14637 $
 * $Author: mokraemer $
 * $Date: 2017-12-05 02:10:46 +0100 (Tue, 05 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_editor_footer{

	private static function fileLocked(we_contents_root $we_doc){
//	user
		$username = f('SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . intval($we_doc->isLockedByUser()));
		//	spaceholder
		$messageTbl = '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:5px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>' .
			sprintf(g_l('alert', '[file_locked_footer]'), $username) .
			we_html_button::create_button(we_html_button::RELOAD, "javascript:WE().layout.weNavigationHistory.navigateReload();") .
			we_html_button::create_button(we_html_button::UNLOCK, "javascript:top.we_cmd('requestUnlock','" . $username . "'," . $we_doc->ID . ",'" . stripTblPrefix($we_doc->Table) . "');");

		echo we_html_tools::getHtmlTop('', '', '', '', we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], $messageTbl)
		));
	}

	private static function fileInWorkspace(){
		$messageTbl = '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:5px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>' .
			g_l('alert', '[' . FILE_TABLE . '][not_im_ws]');

		echo we_html_tools::getHtmlTop('', '', '', '', we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], $messageTbl)));
	}

	private static function fileNoSave(){
		$messageTbl = '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:5px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>' .
			g_l('alert', '[file_no_save_footer]');

		echo we_html_tools::getHtmlTop('', '', '', '', we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], $messageTbl)));
	}

	private static function fileIsRestricted(we_contents_root $we_doc){
		$messageTbl = '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:5px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>' .
			str_replace("<br/>", " ", sprintf(g_l('alert', '[no_perms]'), f('SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . intval($we_doc->CreatorID))));

		echo we_html_tools::getHtmlTop('', '', '', '', we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], $messageTbl)));
	}

	private static function workflow(we_contents_root $we_doc){
		if(we_workflow_utility::isUserInWorkflow($we_doc->ID, $we_doc->Table, $_SESSION['user']['ID']) || we_base_permission::hasPerm(we_base_perm::USER_CAN_PUBLISH)){

			$table = ($_SESSION['weS']['we_mode'] == we_base_constants::MODE_NORMAL ?
				we_workflow_view::showFooterForNormalMode($we_doc, $GLOBALS['showPubl']) :
				($_SESSION['weS']['we_mode'] == we_base_constants::MODE_SEE ?
				we_workflow_view::showFooterForSEEMMode($we_doc, $GLOBALS['showPubl']) : ''));

			$we_form = we_html_element::form(['name' => 'we_form', 'method' => 'post'], $table);

			return we_html_element::body(['id' => 'footerBody'], $we_form);
		}

		$table = '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:16px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>' .
			g_l('modules_workflow', '[doc_in_wf_warning]');

		return we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], $table));
	}

	private static function addDelButton(we_contents_root $we_doc){
		$ctrlElem = self::getControlElement($we_doc, 'button', 'delete'); //	look tag we:controlElement for details
		if(!$ctrlElem || !$ctrlElem['hide']){
			return we_html_button::create_button(we_html_button::FOOTER_TRASH, "javascript:WE().util.showConfirm(window, '', WE().consts.g_l.main.delete_single_confirm_delete+doc.Path,['delete_single_document','',doc.Table,1]);");
		}
		return '';
	}

	private static function addCopyButton(we_contents_root $we_doc){
		$ctrlElem = self::getControlElement($we_doc, 'button', 'copy'); //	look tag we:controlElement for details
		if((!$ctrlElem || !$ctrlElem['hide']) && $we_doc->ID){
			return we_html_button::create_button(we_html_button::FOOTER_COPY, "javascript:we_cmd('cloneDocument');");
		}
		return '';
	}

	/**
	 * @return string
	 * @desc Prints the footer for the normal mode
	 */
	private static function normalMode(we_contents_root $we_doc, $we_transaction, $haspermNew, $showPubl, $hasPermDel){
		$main = [];
		$optional = [];
		if($we_doc->ID){
			//del
			if($hasPermDel){
				$optional[] = self::addDelButton($we_doc);
			}

			//copy
			switch($we_doc->ContentType){
				case we_base_ContentTypes::FOLDER:
				case we_base_ContentTypes::OBJECT:
				case we_base_ContentTypes::TEMPLATE:
				case we_base_ContentTypes::CLASS_FOLDER:
					break;
				default:
					$optional[] = self::addCopyButton($we_doc);
			}

			//new
			switch($we_doc->ContentType){
				case we_base_ContentTypes::TEMPLATE:
					$optional[] = we_html_button::create_button(we_html_button::MAKE_NEW_DOC, "javascript:top.we_cmd('new','" . FILE_TABLE . "','','" . we_base_ContentTypes::WEDOCUMENT . "','','" . $we_doc->ID . "');WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorMakeNewDoc(false);");
					break;
				case we_base_ContentTypes::OBJECT:
					$optional[] = we_html_button::create_button(we_html_button::MAKE_NEW_OBJ, "javascript:top.we_cmd('new','" . OBJECT_FILES_TABLE . "','','" . we_base_ContentTypes::OBJECT_FILE . "','" . $we_doc->ID . "');WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorMakeNewDoc(false);");
					break;
			}

			if($showPubl && $we_doc->Published){
				//	Park button
				$ctrlElem = self::getControlElement($we_doc, 'button', 'unpublish'); //	look tag we:controlElement for details

				if(!$ctrlElem || !$ctrlElem['hide']){
					$main[] = we_html_button::create_button(we_html_button::FOOTER_UNPUBLISH, "javascript:we_cmd('unpublish', '" . $we_transaction . "');");
				}
			}
		}

		switch($we_doc->ContentType){
			case we_base_ContentTypes::WEDOCUMENT:
			case we_base_ContentTypes::OBJECT:
			case we_base_ContentTypes::OBJECT_FILE:
			case we_base_ContentTypes::FOLDER:
			case we_base_ContentTypes::CLASS_FOLDER:
			case we_base_ContentTypes::COLLECTION:
				break;
			default:
				if(!empty($_SESSION['weS']['hasJava']) && we_modules_info::isActive(we_modules_info::EDITOR)){
					$optional[] = (stripos($we_doc->ContentType, 'text/') !== false ?
						we_html_button::create_button('fa:startEditor,fa-lg fa-clipboard', "javascript:editSource();") :
						we_html_button::create_button('fa:startEditor,fa-lg fa-clipboard', "javascript:editFile();")
						);
				}
		}

		//	Save Button
		$ctrlElem = self::getControlElement($we_doc, 'button', 'save'); //	look tag we:controlElement for details
		if(!$ctrlElem || !$ctrlElem['hide']){
			$main[] = we_html_button::create_button(we_html_button::FOOTER_SAVE, "javascript:WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorPublishWhenSave(false);we_save_document();");
		}

		if(we_modules_info::isActive(we_modules_info::WORKFLOW) && $we_doc->IsTextContentDoc && $we_doc->ID){
			//	Workflow button
			$ctrlElem = self::getControlElement($we_doc, 'button', 'workflow'); //	look tag we:controlElement for details

			if(!$ctrlElem || !$ctrlElem['hide']){
				$main[] = we_html_button::create_button(we_html_button::FOOTER_SAVE_TO_WORKFLOW, "javascript:put_in_workflow('" . stripTblPrefix($we_doc->Table) . "');");
			}
		}

		switch($we_doc->ContentType){
			case we_base_ContentTypes::TEMPLATE:
				if(defined('VERSIONING_TEXT_WETMPL') && defined('VERSIONS_CREATE_TMPL') && VERSIONS_CREATE_TMPL && VERSIONING_TEXT_WETMPL){
					$main[] = we_html_button::create_button(we_html_button::FOOTER_SAVE_VERSION, "javascript:WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorPublishWhenSave(true);we_save_document();");
				}

				$main[] = we_html_forms::checkbox('autoRebuild', false, 'autoRebuild', g_l('global', '[we_rebuild_at_save]'), false, "defaultfont", "WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorAutoRebuild((this.checked));");
				break;
			default:
				if($showPubl){
					$ctrlElem = self::getControlElement($we_doc, 'button', 'publish');
					if(!$ctrlElem || !$ctrlElem['hide']){
						$text = we_modules_info::isActive(we_modules_info::SCHEDULER) && we_schedule_scheduler::saveInScheduler($we_doc) ? 'fat:saveInScheduler,fa-lg fa-clock-o' : we_html_button::FOOTER_PUBLISH;
						$main[] = we_html_element::span(['id' => 'publish_' . $we_doc->ID], we_html_button::create_button($text, "javascript:WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorPublishWhenSave(true);we_save_document();"));
					}
				}
		}

		if(($we_doc->IsTextContentDoc/* || $we_doc->IsFolder */) && $haspermNew){
			$ctrlElem = self::getControlElement($we_doc, 'checkbox', 'makeSameDoc');
			if(!$ctrlElem || !$ctrlElem['hide']){
				$main[] = we_html_element::span((($ctrlElem && $ctrlElem['hide'] ) ? ( ['style' => 'display:none'] ) : ['style' => 'display:inline-block']), we_html_forms::checkbox("makeSameDoc", ( $ctrlElem ? $ctrlElem['checked'] : false), 'makeSameDoc', g_l('global', '[we_make_same][' . $we_doc->ContentType . ']'), false, "defaultfont", " WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorMakeSameDoc( (this.checked) ? true : false );", ( $ctrlElem ? $ctrlElem['readonly'] : false)));
			}
		}

		switch($we_doc->ContentType){
			case we_base_ContentTypes::TEMPLATE:
				if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW)){
					$main[] = we_html_forms::checkbox('makeNewDoc', false, "makeNewDoc", g_l('global', '[we_new_doc_after_save]'), false, "defaultfont", "WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorMakeNewDoc( (this.checked) ? true : false );");
				}
				break;
			case we_base_ContentTypes::OBJECT:
				if(we_base_permission::hasPerm(we_base_perm::OBJECTFILE_NEW)){
					$main[] = we_html_forms::checkbox('makeNewDoc', false, 'makeNewDoc', g_l('modules_object', '[we_new_doc_after_save]'), false, "defaultfont", "WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorMakeNewDoc( (this.checked) ? true : false );");
				}
				break;
		}

		return we_html_element::div(['class' => 'footertable'], we_html_element::div(['class' => 'main'], implode('', $main)) . we_html_element::div(['class' => 'optional'], implode('', array_reverse($optional))));
	}

	/**
	 * @return string
	 * @desc prints the footer for the See-Mode
	 */
	private static function SEEMode(we_contents_root $we_doc, $we_transaction, $haspermNew, $showPubl, $hasPermDel){
		$main = $optional = [];

		//First buttons which are always needed
		//	Always button preview
		if($we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_PREVIEW && in_array(we_base_constants::WE_EDITPAGE_PREVIEW, $we_doc->EditPageNrs)){ // first button is always - preview, when exists
			$main[] = we_html_button::create_button(we_html_button::PREVIEW, "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_PREVIEW . ",'" . $we_transaction . "');");
		}

		// variants
		if($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT && in_array(we_base_constants::WE_EDITPAGE_VARIANTS, $we_doc->EditPageNrs) && $we_doc->canHaveVariants(true) && $we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_VARIANTS){ // first button is always - preview, when exists
			$main[] = we_html_button::create_button('shopVariants', "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_VARIANTS . ",'" . $we_transaction . "');");
		}

		//	image-documents have no preview but thumbnailview instead ...
		if($we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_THUMBNAILS && in_array(we_base_constants::WE_EDITPAGE_THUMBNAILS, $we_doc->EditPageNrs)){
			$main[] = we_html_button::create_button('thumbnails', "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_THUMBNAILS . ",'" . $we_transaction . "');");
		}

		//	Button edit !!!
		if($we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_CONTENT && in_array(we_base_constants::WE_EDITPAGE_CONTENT, $we_doc->EditPageNrs)){ // then button "edit"
			$main[] = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_CONTENT . ", '" . $we_transaction . "');");
		}
		//	Button properties
		if(($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_SCHEDULER) && in_array(we_base_constants::WE_EDITPAGE_PROPERTIES, $we_doc->EditPageNrs)){
			if(we_base_permission::hasPerm(we_base_perm::WORK_PROPERTIES_CAN_SEE)){
				$main[] = we_html_button::create_button('properties', "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_PROPERTIES . ", '" . $we_transaction . "');");
			}
		}

		// Button workspace
		if(($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES) && in_array(we_base_constants::WE_EDITPAGE_WORKSPACE, $we_doc->EditPageNrs)){
			$main[] = we_html_button::create_button('workspace_button', "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_WORKSPACE . ", '" . $we_transaction . "');");
		}


		//	Button scheduler
		if(in_array(we_base_constants::WE_EDITPAGE_SCHEDULER, $we_doc->EditPageNrs) && ($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES) &&
			we_modules_info::isActive(we_modules_info::SCHEDULER) && we_base_permission::hasPerm(we_base_perm::SCHEDULER_CAN_SEE)){
			$main[] = we_html_button::create_button('fat:schedule_button,fa-lg fa-clock-o', "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_SCHEDULER . ", '" . $we_transaction . "');");
		}

		//	Button put in workflow
		if(/* $we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_PROPERTIES && */ $we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_SCHEDULER && // then button "workflow"
			we_modules_info::isActive(we_modules_info::WORKFLOW) && $we_doc->IsTextContentDoc && $we_doc->ID){

			$ctrlElem = self::getControlElement($we_doc, 'button', 'workflow'); //	look tag we:controlElement for details

			if(!$ctrlElem || !$ctrlElem['hide']){
				$main[] = we_html_button::create_button(we_html_button::FOOTER_SAVE_TO_WORKFLOW, "javascript:put_in_workflow('" . stripTblPrefix($we_doc->Table) . "');");
			}
		}

		//Special buttons for special EDITPAGE
		//
		//	1. ONLY in PROPERTY page we need the button unpublish
		//
		if($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES && $showPubl && $we_doc->ID && $we_doc->Published){

			//	button unpublish
			$ctrlElem = self::getControlElement($we_doc, 'button', 'unpublish'); //	look tag we:controlElement for details
			if(!$ctrlElem || !$ctrlElem['hide']){
				$main[] = we_html_button::create_button(we_html_button::FOOTER_UNPUBLISH, "javascript:we_cmd('unpublish', '" . $we_transaction . "');");
			}
		}

		//
		//	2. we always need the buttons -> save and publish
		//
		$ctrlElem = self::getControlElement($we_doc, 'button', 'save'); //	look tag we:controlElement for details
		if(!$ctrlElem || !$ctrlElem['hide']){
			$main[] = we_html_button::create_button(we_html_button::FOOTER_SAVE, "javascript:WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorPublishWhenSave(false);we_save_document();");
		}


		//
		// 3. publish when save and make same doc new.
		//
		if($showPubl){
			$ctrlElem = self::getControlElement($we_doc, 'button', 'publish'); //	look tag we:controlElement for details

			if(!($ctrlElem && $ctrlElem['hide'])){

				$main[] = we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorPublishWhenSave(true);we_save_document();");
			}
		}

		if(($we_doc->IsTextContentDoc/* || $we_doc->IsFolder */) && $haspermNew && ($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PREVIEW || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_THUMBNAILS)){

			//	makesamedoc only when not in edit_include-window
			$ctrlElem = self::getControlElement($we_doc, 'checkbox', 'makeSameDoc');

			$showPubl_makeSamNew = ($ctrlElem && $ctrlElem['hide'] ? '<div style="display: hidden;">' : '') .
				we_html_forms::checkbox("makeSameDoc", ( $ctrlElem ? $ctrlElem['checked'] : false), "makeSameDoc", g_l('global', '[we_make_same][' . $we_doc->ContentType . ']'), false, "defaultfont", " WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorMakeSameDoc( (this.checked) ? true : false );", ( $ctrlElem ? $ctrlElem['readonly'] : false)) .
				($ctrlElem && $ctrlElem['hide'] ? '</div>' : '');

			$main[] = $showPubl_makeSamNew;
		}

		//
		//	4. show delete button to delete this document, not in edit_include-window
		//
		if($hasPermDel && $we_doc->ID){
			$optional[] = self::addDelButton($we_doc);
		}
		return we_html_element::div(['class' => 'footertable'], we_html_element::div(['class' => 'main'], implode('', $main)) . we_html_element::div(['class' => 'optional'], implode('', array_reverse($optional))));
	}

	private static function hasNewPerm(we_contents_root $we_doc){
		//	Check permissions for buttons
		switch($we_doc->ContentType){
			case we_base_ContentTypes::HTML:
				return we_base_permission::hasPerm(we_base_perm::WORK_HTML_NEW);

			case we_base_ContentTypes::WEDOCUMENT:
				return we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW);
			case we_base_ContentTypes::OBJECT_FILE:
				return we_base_permission::hasPerm(we_base_perm::OBJECTFILE_NEW);
			case we_base_ContentTypes::FOLDER:
				switch($we_doc->Table){
					case FILE_TABLE:
						return we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_FOLDER_NEW);
					case TEMPLATES_TABLE:
						return we_base_permission::hasPerm(we_base_perm::WORK_TEMPLATES_FOLDER_NEW);
					case OBJECT_FILES_TABLE :
						return we_base_permission::hasPerm(we_base_perm::OBJECTFILE_NEW_FOLDER);
				}
		}
		return false;
	}

	private static function getControlElement(we_contents_root $we_doc, $type, $name){
		if(isset($we_doc->controlElement) && is_array($we_doc->controlElement)){

			return (isset($we_doc->controlElement[$type][$name]) ?
				$we_doc->controlElement[$type][$name] :
				false);
		}
		return false;
	}

	private static function inWorkflow(we_contents_root $doc){
		if(!we_modules_info::isActive(we_modules_info::WORKFLOW) || !$doc->IsTextContentDoc){
			return false;
		}
		return ($doc->ID ? we_workflow_utility::inWorkflow($doc->ID, $doc->Table) : false);
	}

	private static function checkUserAccess(we_contents_root $we_doc){
		switch($we_doc->userHasAccess()){
			case we_contents_root::USER_HASACCESS : //	all is allowed, creator or owner
				break;

			case we_contents_root::FILE_NOT_IN_USER_WORKSPACE : //	file is not in workspace of user
				self::fileInWorkspace();
				exit();

			case we_contents_root::USER_NO_PERM : //	access is restricted and user has no permission
				self::fileIsRestricted($we_doc);
				exit;

			case we_contents_root::FILE_LOCKED : //	file is locked by another user
				self::fileLocked($we_doc);
				exit;

			case we_contents_root::USER_NO_SAVE : //	user has not the right to save the file.
				self::fileNoSave();
				exit;
		}
	}

	public static function show(){
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', we_base_request::_(we_base_request::TRANSACTION, 'we_transaction'), 1);

// init document
		$we_dt = $_SESSION['weS']['we_data'][$we_transaction];
		$we_doc = we_document::initDoc($we_dt);
		self::checkUserAccess($we_doc);

//	preparations of needed vars
		$showPubl = we_base_permission::hasPerm(we_base_perm::USER_CAN_PUBLISH) && $we_doc->userCanSave() && $we_doc->IsTextContentDoc;
		$reloadPage = (bool) (($showPubl || $we_doc->ContentType == we_base_ContentTypes::TEMPLATE) && (!$we_doc->ID));
		$haspermNew = self::hasNewPerm($we_doc);

		switch($we_doc->Table){
			case FILE_TABLE:
				$hasPermDel = ($we_doc->IsFolder && we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_FOLDER_DELETE)) ||
					(!$we_doc->IsFolder && we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_DELETE));
				break;
			case TEMPLATES_TABLE:
				$hasPermDel = ($we_doc->IsFolder && we_base_permission::hasPerm(we_base_perm::WORK_TEMP_FOLDER_DELETE)) ||
					(!$we_doc->IsFolder && we_base_permission::hasPerm(we_base_perm::WORK_TEMPLATE_DELETE));
				break;
			case OBJECT_FILES_TABLE:
				$hasPermDel = (!$we_doc->IsClassFolder && we_base_permission::hasPerm(we_base_perm::OBJECTFILE_DELETE));
				break;
			case OBJECT_TABLE:
				$hasPermDel = we_base_permission::hasPerm(we_base_perm::OBJECT_DELETE);
				break;
			case VFILE_TABLE:
				$hasPermDel = ($we_doc->IsFolder && we_base_permission::hasPerm(we_base_perm::WORK_COLLECTION_FOLDER_DELETE)) ||
					(!$we_doc->IsFolder && we_base_permission::hasPerm(we_base_perm::WORK_COLLECTION_DELETE));
				break;
			default:
				$hasPermDel = false;
		}


		$showGlossaryCheck = 0; /* (!empty($_SESSION['prefs']['force_glossary_check']) &&
		  ( $we_doc->ContentType == we_base_ContentTypes::WEDOCUMENT || $we_doc->ContentType === we_base_ContentTypes::OBJECT_FILE ) ? 1 : 0);
		 */
//	added for we:controlElement type="button" name="save" hide="true"
		$ctrlElem = self::getControlElement($we_doc, 'button', 'save');

		$canWeSave = $we_doc->userCanSave();

		if($canWeSave &&
			(($ctrlElem && $ctrlElem['hide']) ||
			(we_modules_info::isActive(we_modules_info::WORKFLOW) && self::inWorkflow($we_doc) && (!we_workflow_utility::canUserEditDoc($we_doc->ID, $we_doc->Table, $_SESSION['user']['ID'])))
			)){
			$canWeSave = false;
		}

// publish for templates to save in version
		$pass_publish = $canWeSave && ($showPubl || ($we_doc->ContentType == we_base_ContentTypes::TEMPLATE && defined('VERSIONING_TEXT_WETMPL') && defined('VERSIONS_CREATE_TMPL') && VERSIONS_CREATE_TMPL && VERSIONING_TEXT_WETMPL)) ? [
			['publishWhenSave']] : [];

		$doc = [
			'we_transaction' => $we_transaction,
			'ID' => intval($we_doc->ID),
			'Path' => $we_doc->Path,
			'Text' => $we_doc->Text,
			'Table' => $we_doc->Table,
			'contentType' => $we_doc->ContentType,
			'editFilename' => preg_replace('|/' . $we_doc->Filename . '.*$|', $we_doc->Filename . (isset($we_doc->Extension) ? $we_doc->Extension : ''), $we_doc->Path),
			'makeSameDocCheck' => intval(($we_doc->IsTextContentDoc/* || $we_doc->IsFolder */) && $haspermNew && (!self::inWorkflow($we_doc))),
			'isTemplate' => $we_doc->Table == TEMPLATES_TABLE,
			'isFolder' => $we_doc->ContentType == we_base_ContentTypes::FOLDER,
			'isBinary' => $we_doc->isBinary(),
			'classname' => ($we_doc->Published == 0 ? 'notpublished' : (!in_array($we_doc->Table, [TEMPLATES_TABLE, VFILE_TABLE]) && $we_doc->ModDate > $we_doc->Published ? 'changed' : 'published')),
			'_showGlossaryCheck' => $showGlossaryCheck,
			'weCanSave' => $canWeSave,
			'ctrlElem' => 0,
			'reloadOnSave' => $reloadPage,
			'pass_publish' => $pass_publish,
		];

		if(($we_doc->IsTextContentDoc /* || $we_doc->IsFolder */) && $haspermNew && //	$js_permnew
			($_SESSION['weS']['we_mode'] != we_base_constants::MODE_SEE || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT)){ // not in SeeMode or in editmode
			$ctrlElem = self::getControlElement($we_doc, 'checkbox', 'makeSameDoc');
			$doc['ctrlElem'] = ($ctrlElem ? ($ctrlElem['checked'] ? 1 : 2) : 3);
		}

//	Document is in workflow
		if(self::inWorkflow($we_doc)){
			$body = self::workflow($we_doc);
		} else {
			$_SESSION['weS']['seemForOpenDelSelector'] = [
				'ID' => $we_doc->ID,
				'Table' => $we_doc->Table
			];

			if($we_doc->userCanSave()){
				switch($_SESSION['weS']['we_mode']){
					default:
					case we_base_constants::MODE_NORMAL: // open footer for NormalMode
						$content = self::normalMode($we_doc, $we_transaction, $haspermNew, $showPubl, $hasPermDel);
						break;
					case we_base_constants::MODE_SEE: // open footer for SeeMode
						$content = self::SEEMode($we_doc, $we_transaction, $haspermNew, $showPubl, $hasPermDel);
						break;
				}
			} else if($_SESSION['weS']['we_mode'] == we_base_constants::MODE_SEE){

				$noPermTable = new we_html_table(['class' => 'default footertable'], 1, 2, [[
					'<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:10px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>',
					g_l('SEEM', '[no_permission_to_edit_document]')
					]]
				);
				$content = $noPermTable->getHtml();
			} else {
				$content = '';
			}

			$body = we_html_element::body(array_merge([
					'id' => "footerBody",
					'onload' => "we_footerLoaded();",
						], $we_doc->getEditorBodyAttributes(we_contents_root::EDITOR_FOOTER))
					, we_html_element::form([
						'name' => "we_form",
						'action' => '',
						'onsubmit' => (!empty($we_doc->IsClassFolder) ? 'sub();return false;' : '')
						], we_html_element::hidden('sel', $we_doc->ID) .
						$content
			));
		}
		echo we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(JS_DIR . 'we_editor_footer.js', '', ['id' => 'loadVarEditor_footer', 'data-doc' => setDynamicVar($doc)]), $body);
	}

}
