<?php

/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_fileupload{
	private static $maxUploadSizeMB = -1;
	protected $rpcUrl = WEBEDITION_DIR . 'rpc.php?cmd=uploadProcess';
	protected $name = 'weFileSelect';
	protected $maxUploadSizeBytes = 0;
	protected $genericFileNameTemp = '';
	protected $doCommitFile = true;
	protected $isDragAndDrop = true;
	protected $noPermsCT = [];
	protected $typeCondition = [
		'accepted' => [
			'cts' => '',
			'exts4cts' => '',
			'exts' => '',
			'all' => '',
		],
		'forbidden' => [
			'cts' => '',
			'exts4cts' => '',
			'exts' => '',
			'all' => '',
			'allForbiddenByUploader' => ''
		],
	];
	protected $typeConditionArrs = [
		'accepted' => [
			'cts' => [],
			'exts4cts' => [],
			'exts' => [],
			'all' => '',
		],
		'forbidden' => [
			'cts' => [],
			'exts4cts' => [],
			'exts' => [],
			'all' => [],
		]
	];
	protected $editables = ['.jpg', '.png', '.gif']; // TODO: use this in js too
	protected $metaImportables = ['.jpg', '.png'/*, '.pdf'*/]; 
	protected $permToUpload = [
		'images' => false,
		'editables' => false,
		'metaImportables' => false
	];
	// partly obsolete
	protected $imageEditProps = ['parentID' => 0,
		'sameName' => 'rename',
		'importMetadata' => false,
		'isSearchable' => false,
		'thumbnails' => '',
		'categories' => ''
	];

	const CHUNK_SIZE = 128;
	const ON_ERROR_RETURN = true; //obsolete?
	const ON_ERROR_DIE = true; //obsolete?
	const REPLACE_BY_UNIQUEID = '##REPLACE_BY_UNIQUEID##';
	const REPLACE_BY_FILENAME = '##REPLACE_BY_FILENAME##';
	const USE_FILENAME_FROM_UPLOAD = true;

	protected function __construct($name){
		$this->name = $name;
		$this->isDragAndDrop = !(we_base_browserDetect::isIE() && we_base_browserDetect::getIEVersion() < 11) || we_base_browserDetect::isOpera();
		$this->maxUploadSizeBytes = (self::getMaxUploadSizeMB()) * 1048576;
		$this->maxChunkCount = (self::getMaxUploadSizeMB()) * 1024 / self::CHUNK_SIZE;
	}

	public function getCss(){
		return '';
	}

	public function getJs(){
		return '';
	}

	public function setIsDragAndDrop($isDragAndDrop = true){
		$this->isDragAndDrop = $isDragAndDrop;
	}

	public static function isDragAndDrop(){
		return true; //!((we_base_browserDetect::isIE() && we_base_browserDetect::getIEVersion() < 11) || we_base_browserDetect::isOpera());
	}

	public static function getMaxUploadSizeMB(){
		if(self::$maxUploadSizeMB === -1){
			self::$maxUploadSizeMB = intval(defined('FILE_UPLOAD_MAX_UPLOAD_SIZE') ? FILE_UPLOAD_MAX_UPLOAD_SIZE : 0);
		}

		return self::$maxUploadSizeMB;
	}

	public static function getMaxUploadSizeB(){
		return self::getMaxUploadSizeMB() * 1024 * 1024;
	}

	protected function getForbiddenCtsFromPerms(){
		return array_filter([
			(we_base_permission::hasPerm(we_base_perm::WORK_VIDEO_NEW) ? '' : we_base_ContentTypes::FLASH),
			(we_base_permission::hasPerm(we_base_perm::WORK_VIDEO_NEW) ? '' : we_base_ContentTypes::VIDEO),
			(we_base_permission::hasPerm(we_base_perm::WORK_IMAGE_NEW) ? '' : we_base_ContentTypes::IMAGE),
			(we_base_permission::hasPerm(we_base_perm::WORK_OTHER_NEW) ? '' : we_base_ContentTypes::PDF),
			(we_base_permission::hasPerm(we_base_perm::WORK_OTHER_NEW) ? '' : we_base_ContentTypes::APPLICATION),
			(we_base_permission::hasPerm(we_base_perm::WORK_JS_NEW) ? '' : we_base_ContentTypes::JS),
			(we_base_permission::hasPerm(we_base_perm::WORK_HTACCESS_NEW) ? '' : we_base_ContentTypes::HTACCESS),
			(we_base_permission::hasPerm(we_base_perm::WORK_TEXT_NEW) ? '' : we_base_ContentTypes::XML),
			(we_base_permission::hasPerm(we_base_perm::WORK_TEXT_NEW) ? '' : we_base_ContentTypes::TEXT),
			(we_base_permission::hasPerm(we_base_perm::WORK_CSS_NEW) ? '' : we_base_ContentTypes::CSS),
			(we_base_permission::hasPerm(we_base_perm::WORK_HTML_NEW) ? '' : we_base_ContentTypes::HTML),
		]);
	}

	protected function setTypeCondition($field = 'accepted', array $contentTypes = [], array $extensions = []){
		$allForbiddenByUploader = '';

		if($field != 'accepted'){
			$cts = [];
			$extsByCts = [];
			$newExts = [];
			$newCts = [];
			foreach($contentTypes as $ct){
				$ct = strtolower($ct);
				if(in_array($ct, we_base_ContentTypes::getContentTypes(FILE_TABLE, true))){
					$extsByCts = array_unique(is_array(($newExts = we_base_ContentTypes::getExtension($ct))) ? array_merge($extsByCts, $newExts) :
						($newExts ? array_merge($extsByCts, explode(',', trim($newExts, ','))) : $extsByCts));
					$newCts = we_base_ContentTypes::getRealContentTypes($ct);
					$cts = array_unique(empty($newCts) ? array_merge($cts, [$ct]) : array_merge($cts, $newCts, [$ct]));
				}
			}
			$allForbiddenByUploader = implode(',', array_filter(array_unique(array_merge($cts, $extensions, $extsByCts))));
		}

		$contentTypes = $field !== 'accepted' ? array_merge($contentTypes, $this->noPermsCT) : $contentTypes;
		$cts = [];
		$extsByCts = [];
		$newExts = [];
		$newCts = [];
		foreach($contentTypes as $ct){
			$ct = strtolower($ct);
			if(in_array($ct, we_base_ContentTypes::getContentTypes(FILE_TABLE, true))){
				$extsByCts = array_unique(is_array(($newExts = we_base_ContentTypes::getExtension($ct))) ? array_merge($extsByCts, $newExts) :
					($newExts ? array_merge($extsByCts, explode(',', trim($newExts, ','))) : $extsByCts));
				$newCts = we_base_ContentTypes::getRealContentTypes($ct);
				$cts = array_unique(empty($newCts) ? array_merge($cts, [$ct]) : array_merge($cts, $newCts, [$ct]));
			}
		}

		$this->typeConditionArrs[$field] = [
			'cts' => $cts,
			'exts' => $extensions,
			'exts4cts' => $extsByCts,
			'all' => array_filter(array_unique(array_merge($cts, $extensions, $extsByCts)))
		];

		$typeCondition = [
			'cts' => $cts ? ',' . implode(',', $cts) . ',' : '',
			'exts4cts' => $extsByCts ? ',' . implode(',', $extsByCts) . ',' : '',
			'exts' => $extensions ? ',' . strtolower(implode(',', array_unique($extensions))) . ',' : ''
		];
		$typeCondition['all'] = str_replace(',,', ',', $typeCondition['cts'] . $typeCondition['exts4cts'] . $typeCondition['exts']);

		if($field !== 'accepted'){
			$typeCondition['allForbiddenByUploader'] = $allForbiddenByUploader;
		}

		$this->typeCondition[$field] = $typeCondition;
	}

	protected function setUploadPerms(){
		$this->setUploadPerm('editables');
		$this->setUploadPerm('images');
		$this->setUploadPerm('metaImportables');
	}

	protected function setUploadPerm($what){
		switch($what){
			case 'editables':
				$permittedExts = $this->editables;
				break;
			case 'images':
				$permittedExts = we_base_ContentTypes::getExtension(we_base_ContentTypes::IMAGE);
				break;
			case 'metaImportables':
				$permittedExts = $this->metaImportables;
				break;
		}

		if(empty($this->typeConditionArrs['accepted']['all'])){
			$intersect = array_intersect($this->typeConditionArrs['forbidden']['all'], $permittedExts);
			$this->perm[$what] = empty($this->typeConditionArrs['forbidden']['all']) || !empty(array_diff($permittedExts, $intersect));
			return;
		}

		$diff = array_diff($this->typeConditionArrs['accepted']['all'], $this->typeConditionArrs['forbidden']['all']);
		$this->perm[$what] = !empty(array_intersect($diff, $permittedExts));
		return;
	}

	protected function isPermToUpload($what){
		return $this->perm[$what];
	}

	public function setImageEditProps($props){
		if(empty($props) || !is_array($props)){
			return;
		}

		$this->imageEditProps = array_merge($this->imageEditProps, $props);
	}

	public function loadImageEditPropsFromSession(){
		$this->imageEditProps = isset($_SESSION['weS']['we_fileupload']['imageEditProps']) ? $_SESSION['weS']['we_fileupload']['imageEditProps'] : $this->imageEditProps;
	}

	public function saveImageEditPropsInSession(){
		$_SESSION['weS']['we_fileupload']['imageEditProps'] = $this->imageEditProps;
	}

}
