<?php
/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Filter model class for navigation tool
 *
 */
class we_navigation_customerFilter extends we_customer_abstractFilter{
	private $useDocumentFilter = true;

	public function __construct($mode = self::OFF, array $specificCustomers = [], array $blackList = [], array $whiteList = [], array $filter = []){
		parent::__construct($mode, $specificCustomers, $blackList, $whiteList, $filter);
	}

	/**
	 * initialize object with a naviagtion model
	 *
	 * @param we_navigation_navigation $navModel
	 */
	function initByNavModel(we_navigation_navigation $navModel){
		// convert navigation data into data the filter model needs

		$this->setBlackList(isset($navModel->BlackList) ? $navModel->BlackList : []);
		$this->setWhiteList(isset($navModel->WhiteList) ? $navModel->WhiteList : []);
		$this->setSpecificCustomers(isset($navModel->Customers) ? $navModel->Customers : []);
		$this->setFilter($navModel->CustomerFilter);
		$this->setMode($navModel->FilterMode);
		$this->setUseDocumentFilter($navModel->UseDocumentFilter);
	}

	/**
	 * initialize object with a navigation item
	 *
	 * @param we_navigation_item $navItem
	 */
	function initByNavItem(we_navigation_item $navItem){
		$this->setMode($navItem->filtermode);
		if(!empty($navItem->customers['filter'])){
			$this->setFilter($navItem->customers['filter']);
		}
		if(!empty($navItem->customers['blacklist'])){
			$this->setBlackList($navItem->customers['blacklist']);
		}
		if(!empty($navItem->customers['whitelist'])){
			$this->setWhiteList($navItem->customers['whitelist']);
		}
		if(!empty($navItem->customers['id'])){
			$this->setSpecificCustomers($navItem->customers['id']);
		}
	}

	function getUseDocumentFilter(){
		return $this->useDocumentFilter;
	}

	function setUseDocumentFilter($useDocumentFilter){
		$this->useDocumentFilter = $useDocumentFilter;
	}

	static function getUseDocumentFilterFromRequest(){
		return we_base_request::_(we_base_request::BOOL, 'wecf_useDocumentFilter');
	}

	public static function updateByFilter(we_customer_documentFilter $filterObj, $id, $table){
		$mode = $filterObj->getMode();

		$DB_WE = new DB_WE();
		$DB_WE->query('UPDATE ' . NAVIGATION_TABLE . ' SET ' .
			we_database_base::arraySetter([
				'FilterMode' => $mode,
				'Customers' => implode(',', $filterObj->getSpecificCustomers()),
				'CustomerFilter' => we_serialize($filterObj->getFilter(), SERIALIZE_JSON),
				'BlackList' => implode(',', $filterObj->getBlackList()),
				'WhiteList' => implode(',', $filterObj->getWhiteList())
			]) .
			' WHERE UseDocumentFilter=1 AND ' . we_navigation_navigation::getNavCondition($id, $table));
	}

}
