<?php

/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
function _we_tag_ifVar_op($operator, $first, $match){
	switch($operator){
		default:
		case 'equal':
			return $first == $match;
		case 'less':
			return $first < $match;
		case 'less|equal':
			return $first <= $match;
		case 'greater':
			return $first > $match;
		case 'greater|equal':
			return $first >= $match;
		case 'contains':
			return (strpos($first, $match) !== false);
		case 'isin':
			return (strpos($match, $first) !== false);
	}
}

function we_tag_ifVar(array $attribs){
	if(($foo = attributFehltError($attribs, ['name' => false, 'match' => true], __FUNCTION__))){
		echo $foo;
		return false;
	}

	$match_orig = weTag_getAttribute('match', $attribs, '', we_base_request::STRING);
	$type = weTag_getAttribute('type', $attribs, 'document', we_base_request::STRING);
	$operator = weTag_getAttribute('operator', $attribs, 'equal', we_base_request::STRING);

	if(is_bool($match_orig)){
		$size = 1;
	} elseif(is_array($match_orig)){
		$matchArray = $match_orig;
		$size = count($matchArray);
	} else {
		$matchArray = makeArrayFromCSV($match_orig);
		$size = count($matchArray);
		$regs = [];
		if($size == 1 && preg_match('|^\\\\?\$([^\[]+)(\[.*\])?|', $match_orig, $regs)){
			$match_orig = (isset($regs[2]) ?
				getArrayValue($GLOBALS, $regs[1], $regs[2]) :
				(isset($GLOBALS[$regs[1]]) ? $GLOBALS[$regs[1]] : ''));
		}
	}

	$nameOrig = weTag_getAttribute('_name_orig', $attribs, '', we_base_request::STRING);

	switch($type){
		case 'customer' :
		case 'sessionfield' :
			return ($size == 1 && isset($_SESSION['webuser'][$nameOrig]) ?
				_we_tag_ifVar_op($operator, $_SESSION['webuser'][$nameOrig], $match_orig) :
				(isset($_SESSION['webuser'][$nameOrig]) && in_array($_SESSION['webuser'][$nameOrig], $matchArray)));

		case 'global':
			$var = getArrayValue($GLOBALS, null, $nameOrig);
			return ($size == 1 ?
				_we_tag_ifVar_op($operator, $var, $match_orig) :
				(!empty($var) && in_array($var, $matchArray)));

		case 'request':
			$var = getArrayValue($_REQUEST, null, $nameOrig);
			return ($size == 1 ?
				_we_tag_ifVar_op($operator, $var, $match_orig) :
				(!empty($var) && in_array($var, $matchArray)));

		case 'post':
			$var = getArrayValue($_POST, null, $nameOrig);
			return ($size == 1 ?
				_we_tag_ifVar_op($operator, $var, $match_orig) :
				(!empty($var) && in_array($var, $matchArray)));

		case 'get':
			$var = getArrayValue($_GET, null, $nameOrig);
			return ($size == 1 ?
				_we_tag_ifVar_op($operator, $var, $match_orig) :
				(!empty($var) && in_array($var, $matchArray)));

		case 'session':
			$var = getArrayValue($_SESSION, null, $nameOrig);
			return ($size == 1 ?
				_we_tag_ifVar_op($operator, $var, $match_orig) :
				(!empty($var) && in_array($var, $matchArray)));

		case 'property':
			$doc = we_getDocForTag(weTag_getAttribute('doc', $attribs, '', we_base_request::STRING), true);
			if(!isset($doc->$nameOrig)){
				return false;
			}
			$var = $doc->$nameOrig;
			return ($size == 1 ?
				_we_tag_ifVar_op($operator, $var, $match_orig) :
				in_array($var, $matchArray));

		case 'document':
		default:
			$doc = we_getDocForTag(weTag_getAttribute('doc', $attribs, '', we_base_request::STRING), true);
			$val = $doc->getField($attribs, $type, true);

			return ($size == 1 ?
				_we_tag_ifVar_op($operator, $val, $match_orig) :
				in_array($val, $matchArray));
	}
}
